@@ -7,18 +7,20 @@ import logging
 import threading
 import subprocess
 import six
+import base64
 from multiprocessing import Process, Queue
 try:
     from shlex import quote as cmd_quote
 except ImportError:
     from pipes import quote as cmd_quote  # for Python 2.7
 from localstack import config
+from localstack.utils import bootstrap
 from localstack.utils.common import (
-    CaptureOutput, FuncThread, TMP_FILES, short_uid, save_file,
+    CaptureOutput, FuncThread, TMP_FILES, short_uid, save_file, rm_rf, in_docker,
     to_str, run, cp_r, json_safe, get_free_tcp_port)
 from localstack.services.install import INSTALL_PATH_LOCALSTACK_FAT_JAR
 from localstack.utils.aws.dead_letter_queue import lambda_error_to_dead_letter_queue, sqs_error_to_dead_letter_queue
-from localstack.utils.cloudwatch.cloudwatch_util import store_cloudwatch_logs
+from localstack.utils.cloudwatch.cloudwatch_util import store_cloudwatch_logs, cloudwatched
 
 # constants
 LAMBDA_EXECUTOR_JAR = INSTALL_PATH_LOCALSTACK_FAT_JAR
@@ -45,8 +47,6 @@ LAMBDA_RUNTIME_RUBY = 'ruby'
 LAMBDA_RUNTIME_RUBY25 = 'ruby2.5'
 LAMBDA_RUNTIME_PROVIDED = 'provided'
 
-LAMBDA_EVENT_FILE = 'event_file.json'
-
 LAMBDA_SERVER_UNIQUE_PORTS = 500
 LAMBDA_SERVER_PORT_OFFSET = 5000
 
@@ -61,6 +61,9 @@ MAX_CONTAINER_IDLE_TIME_MS = 600 * 1000
 
 EVENT_SOURCE_SQS = 'aws:sqs'
 
+# IP address of main Docker container (lazily initialized)
+DOCKER_MAIN_CONTAINER_IP = None
+
 
 def get_from_event(event, key):
     try:
@@ -89,40 +92,72 @@ def _store_logs(func_details, log_output, invocation_time=None, container_id=Non
     return store_cloudwatch_logs(log_group_name, log_stream_name, log_output, invocation_time)
 
 
+def get_main_endpoint_from_container():
+    global DOCKER_MAIN_CONTAINER_IP
+    if DOCKER_MAIN_CONTAINER_IP is None:
+        DOCKER_MAIN_CONTAINER_IP = False
+        try:
+            if in_docker():
+                DOCKER_MAIN_CONTAINER_IP = bootstrap.get_main_container_ip()
+                LOG.info('Determined main container target IP: %s' % DOCKER_MAIN_CONTAINER_IP)
+        except Exception as e:
+            LOG.info('Unable to get IP address of main Docker container "%s": %s' %
+                (bootstrap.MAIN_CONTAINER_NAME, e))
+    # return main container IP, or fall back to Docker host (bridge IP, or host DNS address)
+    return DOCKER_MAIN_CONTAINER_IP or config.DOCKER_HOST_FROM_CONTAINER
+
+
 class LambdaExecutor(object):
     """ Base class for Lambda executors. Subclasses must overwrite the _execute method """
     def __init__(self):
         # keeps track of each function arn and the last time it was invoked
         self.function_invoke_times = {}
 
+    def _prepare_environment(self, func_details):
+        # setup environment pre-defined variables for docker environment
+        result = func_details.envvars.copy()
+
+        # injecting aws credentials into docker environment if not provided
+        if ('AWS_ACCESS_KEY_ID' not in result.keys() and
+                'AWS_SECRET_ACCESS_KEY' not in result.keys()):
+            result['AWS_ACCESS_KEY_ID'] = 'foobar'
+            result['AWS_SECRET_ACCESS_KEY'] = 'foobar'
+
+        return result
+
     def execute(self, func_arn, func_details, event, context=None, version=None,
             asynchronous=False, callback=None):
         def do_execute(*args):
-            # set the invocation time in milliseconds
-            invocation_time = int(time.time() * 1000)
-            # start the execution
-            raised_error = None
-            result = None
-            dlq_sent = None
-            try:
-                result = self._execute(func_arn, func_details, event, context, version)
-            except Exception as e:
-                raised_error = e
-                if asynchronous:
-                    if get_from_event(event, 'eventSource') == EVENT_SOURCE_SQS:
-                        sqs_queue_arn = get_from_event(event, 'eventSourceARN')
-                        if sqs_queue_arn:
-                            # event source is SQS, send event back to dead letter queue
-                            dlq_sent = sqs_error_to_dead_letter_queue(sqs_queue_arn, event, e)
-                    else:
-                        # event source is not SQS, send back to lambda dead letter queue
-                        lambda_error_to_dead_letter_queue(func_details, event, e)
-                raise e
-            finally:
-                self.function_invoke_times[func_arn] = invocation_time
-                callback and callback(result, func_arn, event, error=raised_error, dlq_sent=dlq_sent)
-            # return final result
-            return result
+
+            @cloudwatched('lambda')
+            def _run(func_arn=None):
+                # set the invocation time in milliseconds
+                invocation_time = int(time.time() * 1000)
+                # start the execution
+                raised_error = None
+                result = None
+                dlq_sent = None
+                try:
+                    result = self._execute(func_arn, func_details, event, context, version)
+                except Exception as e:
+                    raised_error = e
+                    if asynchronous:
+                        if get_from_event(event, 'eventSource') == EVENT_SOURCE_SQS:
+                            sqs_queue_arn = get_from_event(event, 'eventSourceARN')
+                            if sqs_queue_arn:
+                                # event source is SQS, send event back to dead letter queue
+                                dlq_sent = sqs_error_to_dead_letter_queue(sqs_queue_arn, event, e)
+                        else:
+                            # event source is not SQS, send back to lambda dead letter queue
+                            lambda_error_to_dead_letter_queue(func_details, event, e)
+                    raise e
+                finally:
+                    self.function_invoke_times[func_arn] = invocation_time
+                    callback and callback(result, func_arn, event, error=raised_error, dlq_sent=dlq_sent)
+                # return final result
+                return result
+
+            return _run(func_arn=func_arn)
 
         # Inform users about asynchronous mode of the lambda execution.
         if asynchronous:
@@ -202,7 +237,7 @@ class LambdaExecutorContainers(LambdaExecutor):
         lambda_cwd = func_details.cwd
         runtime = func_details.runtime
         handler = func_details.handler
-        environment = func_details.envvars.copy()
+        environment = self._prepare_environment(func_details)
 
         # configure USE_SSL in environment
         if config.USE_SSL:
@@ -215,20 +250,25 @@ class LambdaExecutorContainers(LambdaExecutor):
         event_body = json.dumps(json_safe(event))
         stdin = self.prepare_event(environment, event_body)
 
-        docker_host = config.DOCKER_HOST_FROM_CONTAINER
+        main_endpoint = get_main_endpoint_from_container()
 
-        environment['HOSTNAME'] = docker_host
-        environment['LOCALSTACK_HOSTNAME'] = docker_host
+        environment['LOCALSTACK_HOSTNAME'] = main_endpoint
         environment['_HANDLER'] = handler
+        if os.environ.get('HTTP_PROXY'):
+            environment['HTTP_PROXY'] = os.environ['HTTP_PROXY']
         if func_details.timeout:
             environment['AWS_LAMBDA_FUNCTION_TIMEOUT'] = str(func_details.timeout)
         if context:
             environment['AWS_LAMBDA_FUNCTION_NAME'] = context.function_name
             environment['AWS_LAMBDA_FUNCTION_VERSION'] = context.function_version
             environment['AWS_LAMBDA_FUNCTION_INVOKED_ARN'] = context.invoked_function_arn
+            if hasattr(context, 'client_context'):
+                environment['AWS_LAMBDA_CLIENT_CONTEXT'] = json.dumps(to_str(base64.b64decode(
+                    bytes(context.client_context, encoding='utf8'))))
 
         # custom command to execute in the container
         command = ''
+        events_file = ''
 
         # if running a Java Lambda, set up classpath arguments
         if is_java_lambda(runtime):
@@ -240,10 +280,11 @@ class LambdaExecutorContainers(LambdaExecutor):
                 cp_r(LAMBDA_EXECUTOR_JAR, target_file)
             # TODO cleanup once we have custom Java Docker image
             taskdir = '/var/task'
-            save_file(os.path.join(lambda_cwd, LAMBDA_EVENT_FILE), event_body)
+            events_file = '_lambda.events.%s.json' % short_uid()
+            save_file(os.path.join(lambda_cwd, events_file), event_body)
             classpath = Util.get_java_classpath(target_file)
             command = ("bash -c 'cd %s; java %s -cp \"%s\" \"%s\" \"%s\" \"%s\"'" %
-                (taskdir, java_opts, classpath, LAMBDA_EXECUTOR_CLASS, handler, LAMBDA_EVENT_FILE))
+                (taskdir, java_opts, classpath, LAMBDA_EXECUTOR_CLASS, handler, events_file))
 
         # accept any self-signed certificates for outgoing calls from the Lambda
         if is_nodejs_runtime(runtime):
@@ -256,6 +297,9 @@ class LambdaExecutorContainers(LambdaExecutor):
         LOG.info('Running lambda cmd: %s' % cmd)
         result = self.run_lambda_executor(cmd, stdin, env_vars=environment, func_details=func_details)
 
+        # clean up events file
+        events_file and os.path.exists(events_file) and rm_rf(events_file)
+
         return result
 
 
@@ -298,13 +342,9 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         # determine files to be copied into the container
         copy_command = ''
         docker_cmd = self._docker_cmd()
-        event_file = os.path.join(lambda_cwd, LAMBDA_EVENT_FILE)
         if not has_been_invoked_before and config.LAMBDA_REMOTE_DOCKER:
             # if this is the first invocation: copy the entire folder into the container
             copy_command = '%s cp "%s/." "%s:/var/task";' % (docker_cmd, lambda_cwd, container_info.name)
-        elif os.path.exists(event_file):
-            # otherwise, copy only the event file if it exists
-            copy_command = '%s cp "%s" "%s:/var/task";' % (docker_cmd, event_file, container_info.name)
 
         cmd = (
             '%s'
@@ -362,7 +402,8 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 
                 mount_volume = not config.LAMBDA_REMOTE_DOCKER
                 lambda_cwd_on_host = Util.get_host_path_for_path_in_docker(lambda_cwd)
-                lambda_cwd_on_host = Util.format_windows_path(lambda_cwd)
+                if (':' in lambda_cwd and '\\' in lambda_cwd):
+                    lambda_cwd_on_host = Util.format_windows_path(lambda_cwd_on_host)
                 mount_volume_str = '-v "%s":/var/task' % lambda_cwd_on_host if mount_volume else ''
 
                 # Create and start the container
@@ -650,7 +691,7 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
 class LambdaExecutorLocal(LambdaExecutor):
     def _execute(self, func_arn, func_details, event, context=None, version=None):
         lambda_cwd = func_details.cwd
-        environment = func_details.envvars.copy()
+        environment = self._prepare_environment(func_details)
 
         # execute the Lambda function in a forked sub-process, sync result via queue
         queue = Queue()
@@ -687,7 +728,7 @@ class LambdaExecutorLocal(LambdaExecutor):
         handler = func_details.handler
         opts = config.LAMBDA_JAVA_OPTS if config.LAMBDA_JAVA_OPTS else ''
         event_file = EVENT_FILE_PATTERN.replace('*', short_uid())
-        save_file(event_file, json.dumps(event))
+        save_file(event_file, json.dumps(json_safe(event)))
         TMP_FILES.append(event_file)
         class_name = handler.split('::')[0]
         classpath = '%s:%s:%s' % (main_file, Util.get_java_classpath(main_file), LAMBDA_EXECUTOR_JAR)
@@ -726,6 +767,7 @@ class Util:
         temp = path.replace(':', '').replace('\\', '/')
         if len(temp) >= 1 and temp[:1] != '/':
             temp = '/' + temp
+        temp = '%s%s' % (config.WINDOWS_DOCKER_MOUNT_PREFIX, temp)
         return temp
 
     @classmethod
@@ -758,13 +800,15 @@ class Util:
         """
         entries = ['.']
         base_dir = os.path.dirname(archive)
-        for pattern in ['%s/*.jar', '%s/lib/*.jar', '%s/*.zip']:
+        for pattern in ['%s/*.jar', '%s/lib/*.jar', '%s/java/lib/*.jar', '%s/*.zip']:
             for entry in glob.glob(pattern % base_dir):
                 if os.path.realpath(archive) != os.path.realpath(entry):
                     entries.append(os.path.relpath(entry, base_dir))
         # make sure to append the localstack-utils.jar at the end of the classpath
         # https://github.com/localstack/localstack/issues/1160
         entries.append(os.path.relpath(archive, base_dir))
+        entries.append('*.jar')
+        entries.append('java/lib/*.jar')
         result = ':'.join(entries)
         return result
 
