@@ -8,14 +8,16 @@ import logging
 import traceback
 import boto3
 import subprocess
+from moto import core as moto_core
 from requests.models import Response
 from localstack import constants, config
 from localstack.constants import (
-    ENV_DEV, LOCALSTACK_VENV_FOLDER, ENV_INTERNAL_TEST_RUN, LOCALSTACK_INFRA_PROCESS, DEFAULT_SERVICE_PORTS)
+    ENV_DEV, LOCALSTACK_VENV_FOLDER, LOCALSTACK_INFRA_PROCESS, DEFAULT_SERVICE_PORTS)
 from localstack.utils import common, persistence
-from localstack.utils.common import (TMP_THREADS, run, get_free_tcp_port, is_linux,
-    FuncThread, ShellCommandThread, get_service_protocol, in_docker, is_port_open, sleep_forever)
+from localstack.utils.common import (TMP_THREADS, run, get_free_tcp_port, is_linux, start_thread,
+    ShellCommandThread, get_service_protocol, in_docker, is_port_open, sleep_forever, print_debug)
 from localstack.utils.server import multiserver
+from localstack.utils.testutil import is_local_test_mode
 from localstack.utils.bootstrap import (
     setup_logging, is_debug, canonicalize_api_names, load_plugins, in_ci)
 from localstack.utils.analytics import event_publisher
@@ -24,7 +26,7 @@ from localstack.services.es import es_api
 from localstack.services.plugins import SERVICE_PLUGINS, record_service_health, check_infra
 from localstack.services.firehose import firehose_api
 from localstack.services.awslambda import lambda_api
-from localstack.services.generic_proxy import GenericProxy, GenericProxyHandler, ProxyListener
+from localstack.services.generic_proxy import GenericProxyHandler, ProxyListener, start_proxy_server
 from localstack.services.dynamodbstreams import dynamodbstreams_api
 
 # flag to indicate whether signal handlers have been set up already
@@ -36,6 +38,9 @@ DEFAULT_BACKEND_HOST = '127.0.0.1'
 # set up logger
 LOG = logging.getLogger(__name__)
 
+# fix moto account ID - note: keep this at the top level here
+moto_core.ACCOUNT_ID = constants.TEST_AWS_ACCOUNT_ID
+
 
 # -----------------------
 # CONFIG UPDATE BACKDOOR
@@ -102,6 +107,11 @@ def start_route53(port=None, asynchronous=False):
     return start_moto_server('route53', port, name='Route53', asynchronous=asynchronous)
 
 
+def start_acm(port=None, asynchronous=False):
+    port = port or config.PORT_ACM
+    return start_moto_server('acm', port, name='ACM', asynchronous=asynchronous)
+
+
 def start_ses(port=None, asynchronous=False):
     port = port or config.PORT_SES
     return start_moto_server('ses', port, name='SES', asynchronous=asynchronous)
@@ -239,17 +249,15 @@ def start_proxy_for_service(service_name, port, backend_port, update_listener, q
 
 def start_proxy(port, backend_url, update_listener=None, quiet=False, params={}, use_ssl=None):
     use_ssl = config.USE_SSL if use_ssl is None else use_ssl
-    proxy_thread = GenericProxy(port=port, forward_url=backend_url,
-        ssl=use_ssl, update_listener=update_listener, quiet=quiet, params=params)
-    proxy_thread.start()
-    TMP_THREADS.append(proxy_thread)
+    proxy_thread = start_proxy_server(port=port, forward_url=backend_url,
+        use_ssl=use_ssl, update_listener=update_listener, quiet=quiet, params=params)
     return proxy_thread
 
 
 def start_moto_server(key, port, name=None, backend_port=None, asynchronous=False, update_listener=None):
     if not name:
         name = key
-    print('Starting mock %s service in %s ports %s (recommended) and %s (deprecated)...' % (
+    print('Starting mock %s service on %s ports %s (recommended) and %s (deprecated)...' % (
         name, get_service_protocol(), config.EDGE_PORT, port))
     if not backend_port and (config.USE_SSL or update_listener):
         backend_port = get_free_tcp_port()
@@ -269,18 +277,16 @@ def start_moto_server_separate(key, port, name=None, backend_port=None, asynchro
 
 
 def start_local_api(name, port, method, asynchronous=False):
-    print('Starting mock %s service in %s ports %s (recommended) and %s (deprecated)...' % (
+    print('Starting mock %s service on %s ports %s (recommended) and %s (deprecated)...' % (
         name, get_service_protocol(), config.EDGE_PORT, port))
     if asynchronous:
-        thread = FuncThread(method, port, quiet=True)
-        thread.start()
-        TMP_THREADS.append(thread)
+        thread = start_thread(method, port, quiet=True)
         return thread
     else:
         method(port)
 
 
-def stop_infra():
+def stop_infra(debug=False):
     if common.INFRA_STOPPED:
         return
     common.INFRA_STOPPED = True
@@ -288,8 +294,11 @@ def stop_infra():
     event_publisher.fire_event(event_publisher.EVENT_STOP_INFRA)
 
     generic_proxy.QUIET = True
+    print_debug('[shutdown] Cleaning up files ...', debug)
     common.cleanup(files=True, quiet=True)
-    common.cleanup_resources()
+    print_debug('[shutdown] Cleaning up resources ...', debug)
+    common.cleanup_resources(debug=debug)
+    print_debug('[shutdown] Cleaning up Lambda resources ...', debug)
     lambda_api.cleanup()
     time.sleep(2)
     # TODO: optimize this (takes too long currently)
@@ -365,7 +374,7 @@ def do_start_infra(asynchronous, apis, is_in_docker):
     os.environ['AWS_REGION'] = config.DEFAULT_REGION
     os.environ['ENV'] = ENV_DEV
     # register signal handlers
-    if not os.environ.get(ENV_INTERNAL_TEST_RUN):
+    if not is_local_test_mode():
         register_signal_handlers()
     # make sure AWS credentials are configured, otherwise boto3 bails on us
     check_aws_credentials()
