@@ -6,24 +6,23 @@ from moto.sqs.models import Message, TRANSPORT_TYPE_ENCODINGS
 from six.moves.urllib import parse as urlparse
 from six.moves.urllib.parse import urlencode
 from requests.models import Request
-from localstack import config
+from localstack import config, constants
 from localstack.config import HOSTNAME_EXTERNAL, SQS_PORT_EXTERNAL
 from localstack.utils.aws import aws_stack
 from localstack.services.sns import sns_listener
-from localstack.utils.common import to_str, clone
+from localstack.utils.common import to_str, clone, path_from_url, get_service_protocol
 from localstack.utils.analytics import event_publisher
+from localstack.services.install import SQS_BACKEND_IMPL
 from localstack.utils.persistence import PersistingProxyListener
 from localstack.services.awslambda import lambda_api
-from localstack.utils.aws.aws_responses import requests_response, make_error
+from localstack.utils.aws.aws_responses import requests_response, make_requests_error
 
-XMLNS_SQS = 'http://queue.amazonaws.com/doc/2012-11-05/'
+API_VERSION = '2012-11-05'
+XMLNS_SQS = 'http://queue.amazonaws.com/doc/%s/' % API_VERSION
 
-# list of valid attribute names, and names not supported by the backend (elasticmq)
-VALID_ATTRIBUTE_NAMES = ['DelaySeconds', 'MaximumMessageSize', 'MessageRetentionPeriod',
-                         'ReceiveMessageWaitTimeSeconds', 'RedrivePolicy', 'VisibilityTimeout',
-                         'KmsMasterKeyId', 'KmsDataKeyReusePeriodSeconds',
-                         'CreatedTimestamp', 'LastModifiedTimestamp',
-                         'ApproximateNumberOfMessages', 'ApproximateNumberOfMessagesNotVisible']
+# Valid unicode values: #x9 | #xA | #xD | #x20 to #xD7FF | #xE000 to #xFFFD | #x10000 to #x10FFFF
+# https://docs.aws.amazon.com/AWSSimpleQueueService/latest/APIReference/API_SendMessage.html
+MSG_CONTENT_REGEX = '^[\u0009\u000A\u0020-\uD7FF\uE000-\uFFFD\U00010000-\U0010FFFF]*$'
 
 UNSUPPORTED_ATTRIBUTE_NAMES = [
     # elasticmq store 'FifoQueue', 'ContentBasedDeduplication' as queue's properties
@@ -41,11 +40,14 @@ QUEUE_ATTRIBUTES = {}
 def parse_request_data(method, path, data):
     """ Extract request data either from query string (for GET) or request body (for POST). """
     if method == 'POST':
-        return urlparse.parse_qs(to_str(data))
+        result = urlparse.parse_qs(to_str(data))
     elif method == 'GET':
         parsed_path = urlparse.urlparse(path)
-        return urlparse.parse_qs(parsed_path.query)
-    return {}
+        result = urlparse.parse_qs(parsed_path.query)
+    else:
+        return {}
+    result = dict([(k, v[0]) for k, v in result.items()])
+    return result
 
 
 # Format attributes as a list. Example input:
@@ -60,8 +62,8 @@ def _format_attributes(req_data):
         key2 = 'Attribute.%s.Value' % i
         if key1 not in req_data:
             break
-        key_name = req_data[key1][0]
-        key_value = (req_data.get(key2) or [''])[0]
+        key_name = req_data[key1]
+        key_value = req_data.get(key2) or ''
         result[key_name] = key_value
     return result
 
@@ -72,7 +74,7 @@ def _format_attributes_names(req_data):
         key = 'AttributeName.%s' % i
         if key not in req_data:
             break
-        result.add(req_data[key][0])
+        result.add(req_data[key])
     return result
 
 
@@ -88,6 +90,9 @@ def _get_attributes_forward_request(method, path, headers, req_data, forward_att
 
 
 def _set_queue_attributes(queue_url, req_data):
+    # TODO remove this function if we stop using ElasticMQ entirely
+    if SQS_BACKEND_IMPL != 'elasticmq':
+        return
     attrs = _format_attributes(req_data)
     # select only the attributes in UNSUPPORTED_ATTRIBUTE_NAMES
     local_attrs = {}
@@ -109,35 +114,10 @@ def _set_queue_attributes(queue_url, req_data):
     return forward_attrs
 
 
-def _fire_event(req_data, response):
-    action = req_data.get('Action', [None])[0]
-    event_type = None
-    queue_url = None
-    if action == 'CreateQueue':
-        event_type = event_publisher.EVENT_SQS_CREATE_QUEUE
-        response_data = xmltodict.parse(response.content)
-        if 'CreateQueueResponse' in response_data:
-            queue_url = response_data['CreateQueueResponse']['CreateQueueResult']['QueueUrl']
-    elif action == 'DeleteQueue':
-        event_type = event_publisher.EVENT_SQS_DELETE_QUEUE
-        queue_url = req_data.get('QueueUrl', [None])[0]
-
-    if event_type and queue_url:
-        event_publisher.fire_event(event_type, payload={'u': event_publisher.get_hash(queue_url)})
-
-
-def _queue_url(path, req_data, headers):
-    queue_url = req_data.get('QueueUrl')
-    if queue_url:
-        return queue_url[0]
-    url = config.TEST_SQS_URL
-    if headers.get('Host'):
-        url = 'http%s://%s' % ('s' if config.USE_SSL else '', headers['Host'])
-    queue_url = '%s%s' % (url, path.partition('?')[0])
-    return queue_url
-
-
 def _add_queue_attributes(path, req_data, content_str, headers):
+    # TODO remove this function if we stop using ElasticMQ entirely
+    if SQS_BACKEND_IMPL != 'elasticmq':
+        return content_str
     flags = re.MULTILINE | re.DOTALL
     queue_url = _queue_url(path, req_data, headers)
     requested_attributes = _format_attributes_names(req_data)
@@ -152,6 +132,34 @@ def _add_queue_attributes(path, req_data, content_str, headers):
     return content_str
 
 
+def _fire_event(req_data, response):
+    action = req_data.get('Action')
+    event_type = None
+    queue_url = None
+    if action == 'CreateQueue':
+        event_type = event_publisher.EVENT_SQS_CREATE_QUEUE
+        response_data = xmltodict.parse(response.content)
+        if 'CreateQueueResponse' in response_data:
+            queue_url = response_data['CreateQueueResponse']['CreateQueueResult']['QueueUrl']
+    elif action == 'DeleteQueue':
+        event_type = event_publisher.EVENT_SQS_DELETE_QUEUE
+        queue_url = req_data.get('QueueUrl')
+
+    if event_type and queue_url:
+        event_publisher.fire_event(event_type, payload={'u': event_publisher.get_hash(queue_url)})
+
+
+def _queue_url(path, req_data, headers):
+    queue_url = req_data.get('QueueUrl')
+    if queue_url:
+        return queue_url
+    url = config.TEST_SQS_URL
+    if headers.get('Host'):
+        url = '%s://%s' % (get_service_protocol(), headers['Host'])
+    queue_url = '%s%s' % (url, path.partition('?')[0])
+    return queue_url
+
+
 def _list_dead_letter_source_queues(queues, queue_url):
     dead_letter_source_queues = []
     for k, v in queues.items():
@@ -187,9 +195,17 @@ def format_list_dl_source_queues_response(queues):
 # extract the external port used by the client to make the request
 def get_external_port(headers, request_handler):
     host = headers.get('Host', '')
+
+    if not host:
+        forwarded = headers.get('X-Forwarded-For', '').split(',')
+        host = forwarded[-2] if len(forwarded) > 2 else forwarded[-1]
+
     if ':' in host:
         return int(host.split(':')[1])
 
+    if not request_handler or not request_handler.proxy:
+        return config.PORT_SQS
+
     # If we cannot find the Host header, then fall back to the port of the proxy.
     # (note that this could be incorrect, e.g., if running in Docker with a host port that
     # is different from the internal container port, but there is not much else we can do.)
@@ -203,6 +219,11 @@ def validate_empty_message_batch(data, req_data):
     return False
 
 
+def is_sqs_queue_url(url):
+    path = path_from_url(url).partition('?')[0]
+    return re.match(r'^/(queue|%s)/[a-zA-Z0-9_-]+$' % constants.TEST_AWS_ACCOUNT_ID, path)
+
+
 class ProxyListenerSQS(PersistingProxyListener):
 
     def api_name(self):
@@ -213,15 +234,31 @@ class ProxyListenerSQS(PersistingProxyListener):
             return 200
 
         req_data = parse_request_data(method, path, data)
-        if req_data:
-            action = req_data.get('Action', [None])[0]
 
-            if action == 'SetQueueAttributes':
+        if is_sqs_queue_url(path) and method == 'GET':
+            if not headers.get('Authorization'):
+                headers['Authorization'] = aws_stack.mock_aws_request_headers(service='sqs')['Authorization']
+            method = 'POST'
+            req_data = {'Action': 'GetQueueUrl', 'Version': API_VERSION, 'QueueName': path.split('/')[-1]}
+
+        if req_data:
+            action = req_data.get('Action')
+
+            if action in ('SendMessage', 'SendMessageBatch') and SQS_BACKEND_IMPL == 'moto':
+                # check message contents
+                for key, value in req_data.items():
+                    if not re.match(MSG_CONTENT_REGEX, str(value)):
+                        return make_requests_error(code=400, code_string='InvalidMessageContents',
+                            message='Message contains invalid characters')
+
+            elif action == 'SetQueueAttributes':
+                # TODO remove this function if we stop using ElasticMQ entirely
                 queue_url = _queue_url(path, req_data, headers)
-                forward_attrs = _set_queue_attributes(queue_url, req_data)
-                if len(req_data) != len(forward_attrs):
-                    # make sure we only forward the supported attributes to the backend
-                    return _get_attributes_forward_request(method, path, headers, req_data, forward_attrs)
+                if SQS_BACKEND_IMPL == 'elasticmq':
+                    forward_attrs = _set_queue_attributes(queue_url, req_data)
+                    if len(req_data) != len(forward_attrs):
+                        # make sure we only forward the supported attributes to the backend
+                        return _get_attributes_forward_request(method, path, headers, req_data, forward_attrs)
 
             elif action == 'DeleteQueue':
                 queue_url = _queue_url(path, req_data, headers)
@@ -229,11 +266,12 @@ class ProxyListenerSQS(PersistingProxyListener):
                 sns_listener.unsubscribe_sqs_queue(queue_url)
 
             elif action == 'ListDeadLetterSourceQueues':
+                # TODO remove this function if we stop using ElasticMQ entirely
                 queue_url = _queue_url(path, req_data, headers)
-                headers = {'content-type': 'application/xhtml+xml'}
-                content_str = _list_dead_letter_source_queues(QUEUE_ATTRIBUTES, queue_url)
-
-                return requests_response(content_str, headers=headers)
+                if SQS_BACKEND_IMPL == 'elasticmq':
+                    headers = {'content-type': 'application/xhtml+xml'}
+                    content_str = _list_dead_letter_source_queues(QUEUE_ATTRIBUTES, queue_url)
+                    return requests_response(content_str, headers=headers)
 
             if 'QueueName' in req_data:
                 encoded_data = urlencode(req_data, doseq=True) if method == 'POST' else ''
@@ -259,8 +297,8 @@ class ProxyListenerSQS(PersistingProxyListener):
             return
 
         region_name = aws_stack.get_region()
-        req_data = urlparse.parse_qs(to_str(data))
-        action = req_data.get('Action', [None])[0]
+        req_data = parse_request_data(method, path, data)
+        action = req_data.get('Action')
         content_str = content_str_original = to_str(response.content)
 
         if response.status_code >= 400:
@@ -273,7 +311,7 @@ class ProxyListenerSQS(PersistingProxyListener):
             content_str = _add_queue_attributes(path, req_data, content_str, headers)
 
         # patch the response and return the correct endpoint URLs / ARNs
-        if action in ('CreateQueue', 'GetQueueUrl', 'ListQueues', 'GetQueueAttributes'):
+        if action in ('CreateQueue', 'GetQueueUrl', 'ListQueues', 'GetQueueAttributes', 'ListDeadLetterSourceQueues'):
             if config.USE_SSL and '<QueueUrl>http://' in content_str:
                 # return https://... if we're supposed to use SSL
                 content_str = re.sub(r'<QueueUrl>\s*http://', r'<QueueUrl>https://', content_str)
@@ -282,18 +320,23 @@ class ProxyListenerSQS(PersistingProxyListener):
             content_str = re.sub(r'<QueueUrl>\s*([a-z]+)://[^<]*:([0-9]+)/([^<]*)\s*</QueueUrl>',
                                  r'<QueueUrl>\1://%s:%s/\3</QueueUrl>' % (HOSTNAME_EXTERNAL, external_port),
                                  content_str)
+            # encode account ID in queue URL
+            content_str = re.sub(r'<QueueUrl>\s*([a-z]+)://([^/]+)/queue/([^<]*)\s*</QueueUrl>',
+                                 r'<QueueUrl>\1://\2/%s/\3</QueueUrl>' % constants.TEST_AWS_ACCOUNT_ID,
+                                 content_str)
             # fix queue ARN
             content_str = re.sub(r'<([a-zA-Z0-9]+)>\s*arn:aws:sqs:elasticmq:([^<]+)</([a-zA-Z0-9]+)>',
                                  r'<\1>arn:aws:sqs:%s:\2</\3>' % region_name, content_str)
 
             if action == 'CreateQueue':
                 queue_url = re.match(r'.*<QueueUrl>(.*)</QueueUrl>', content_str, re.DOTALL).group(1)
-                _set_queue_attributes(queue_url, req_data)
+                if SQS_BACKEND_IMPL == 'elasticmq':
+                    _set_queue_attributes(queue_url, req_data)
 
         elif action == 'SendMessageBatch':
             if validate_empty_message_batch(data, req_data):
                 msg = 'There should be at least one SendMessageBatchRequestEntry in the request.'
-                return make_error(code=404, code_string='EmptyBatchRequest', message=msg)
+                return make_requests_error(code=404, code_string='EmptyBatchRequest', message=msg)
 
         # instruct listeners to fetch new SQS message
         if action in ('SendMessage', 'SendMessageBatch'):
@@ -305,6 +348,7 @@ class ProxyListenerSQS(PersistingProxyListener):
             return requests_response(content_str, headers=response.headers, status_code=response.status_code)
 
     @classmethod
+    # TODO still needed? (can probably be removed)
     def get_message_attributes_md5(cls, req_data):
         req_data = clone(req_data)
         orig_types = {}
@@ -313,19 +357,21 @@ class ProxyListenerSQS(PersistingProxyListener):
             # not supported: Keep track of the original data type, and temporarily change
             # it to the short form (e.g., 'Number'), before changing it back again.
             if key.endswith('DataType'):
-                parts = entry[0].split('.')
+                parts = entry.split('.')
                 if len(parts) > 2:
                     short_type_name = parts[0]
-                    full_type_name = req_data[key][0]
+                    full_type_name = entry
                     attr_num = key.split('.')[1]
-                    attr_name = req_data['MessageAttribute.%s.Name' % attr_num][0]
+                    attr_name = req_data['MessageAttribute.%s.Name' % attr_num]
                     orig_types[attr_name] = full_type_name
                     req_data[key] = [short_type_name]
                     if full_type_name not in TRANSPORT_TYPE_ENCODINGS:
                         TRANSPORT_TYPE_ENCODINGS[full_type_name] = TRANSPORT_TYPE_ENCODINGS[short_type_name]
 
+        # moto parse_message_attributes(..) expects params to be passed as dict of lists
+        req_data_lists = dict([(k, [v]) for k, v in req_data.items()])
         moto_message = Message('dummy_msg_id', 'dummy_body')
-        moto_message.message_attributes = parse_message_attributes(req_data)
+        moto_message.message_attributes = parse_message_attributes(req_data_lists)
         for key, data_type in orig_types.items():
             moto_message.message_attributes[key]['data_type'] = data_type
         message_attr_hash = moto_message.attribute_md5
