@@ -367,6 +367,12 @@ RESOURCE_TO_FUNCTION = {
             'defaults': {
                 'Role': 'test_role'
             }
+        },
+        'delete': {
+            'function': 'delete_function',
+            'parameters': {
+                'FunctionName': 'PhysicalResourceId'
+            }
         }
     },
     'Lambda::Version': {
@@ -801,6 +807,15 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             topics = aws_stack.connect_to_service('sns').list_topics()
             result = list(filter(lambda item: item['TopicArn'] == resource_id, topics.get('Topics', [])))
             return result[0] if result else None
+        elif resource_type == 'SNS::Subscription':
+            topic_arn = resource_props.get('TopicArn')
+            topic_arn = resolve_refs_recursively(stack_name, topic_arn, resources)
+            subs = aws_stack.connect_to_service('sns').list_subscriptions_by_topic(TopicArn=topic_arn)
+            result = [sub for sub in subs['Subscriptions'] if
+                resource_props.get('Protocol') == sub['Protocol'] and
+                resource_props.get('Endpoint') == sub['Endpoint']]
+            # TODO: use get_subscription_attributes to compare FilterPolicy
+            return result[0] if result else None
         elif resource_type == 'S3::Bucket':
             bucket_name = resource_props.get('BucketName') or resource_id
             bucket_name = resolve_refs_recursively(stack_name, bucket_name, resources)
@@ -1077,9 +1092,22 @@ def deploy_resource(resource_id, resources, stack_name):
 
 
 def delete_resource(resource_id, resources, stack_name):
-    for res_id, res in resources.items():
-        if res['ResourceType'] == 'AWS::S3::Bucket':
-            s3_listener.remove_bucket_notification(res['PhysicalResourceId'])
+    res = resources[resource_id]
+    if res['ResourceType'] == 'AWS::S3::Bucket':
+        s3_listener.remove_bucket_notification(res['PhysicalResourceId'])
+
+    if res['ResourceType'] == 'AWS::IAM::Role':
+        role_name = res['PhysicalResourceId']
+
+        iam_client = aws_stack.connect_to_service('iam')
+        rs = iam_client.list_role_policies(
+            RoleName=role_name
+        )
+        for policy in rs['PolicyNames']:
+            iam_client.delete_role_policy(
+                RoleName=role_name,
+                PolicyName=policy
+            )
 
     return execute_resource_action(resource_id, resources, stack_name, ACTION_DELETE)
 
@@ -1202,7 +1230,7 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
             api_id = resolve_refs_recursively(stack_name, resource_props['RestApiId'], resources)
             res_id = resolve_refs_recursively(stack_name, resource_props['ResourceId'], resources)
             apigateway.put_method_response(restApiId=api_id, resourceId=res_id,
-                httpMethod=resource_props['HttpMethod'], statusCode=response['StatusCode'],
+                httpMethod=resource_props['HttpMethod'], statusCode=str(response['StatusCode']),
                 responseParameters=response.get('ResponseParameters', {}))
 
     elif resource_type == 'SNS::Topic':
@@ -1237,7 +1265,6 @@ def delete_stack(stack_name, stack_resources):
 # --------
 # Util methods for analyzing resource dependencies
 # --------
-
 def is_deployable_resource(resource):
     resource_type = get_resource_type(resource)
     entry = RESOURCE_TO_FUNCTION.get(resource_type)
