@@ -1,30 +1,33 @@
 import io
 import os
 import re
+import pwd
+import grp
 import sys
-import pty
 import json
 import uuid
 import time
 import glob
 import base64
-import select
 import socket
 import hashlib
 import decimal
 import logging
+import tarfile
 import zipfile
 import binascii
+import calendar
 import tempfile
 import threading
-import traceback
 import subprocess
 import six
 import shutil
 import requests
+import dns.resolver
+import functools
 from io import BytesIO
 from contextlib import closing
-from datetime import datetime
+from datetime import datetime, date
 from six import with_metaclass
 from six.moves import cStringIO as StringIO
 from six.moves.urllib.parse import urlparse
@@ -32,6 +35,8 @@ from multiprocessing.dummy import Pool
 from localstack import config
 from localstack.config import DEFAULT_ENCODING
 from localstack.constants import ENV_DEV
+from localstack.utils import bootstrap
+from localstack.utils.bootstrap import FuncThread
 
 # arrays for temporary files and resources
 TMP_FILES = []
@@ -41,14 +46,13 @@ TMP_PROCESSES = []
 # cache clean variables
 CACHE_CLEAN_TIMEOUT = 60 * 5
 CACHE_MAX_AGE = 60 * 60
-CACHE_FILE_PATTERN = os.path.join(tempfile.gettempdir(), 'cache.*.json')
+CACHE_FILE_PATTERN = os.path.join(tempfile.gettempdir(), '_random_dir_', 'cache.*.json')
 last_cache_clean_time = {'time': 0}
 mutex_clean = threading.Semaphore(1)
-mutex_popen = threading.Semaphore(1)
 
 # misc. constants
 TIMESTAMP_FORMAT = '%Y-%m-%dT%H:%M:%S'
-TIMESTAMP_FORMAT_MILLIS = '%Y-%m-%dT%H:%M:%S.%fZ'
+TIMESTAMP_FORMAT_MICROS = '%Y-%m-%dT%H:%M:%S.%fZ'
 CODEC_HANDLER_UNDERSCORE = 'underscore'
 
 # chunk size for file downloads
@@ -60,6 +64,12 @@ LOG = logging.getLogger(__name__)
 # flag to indicate whether we've received and processed the stop signal
 INFRA_STOPPED = False
 
+# generic cache object
+CACHE = {}
+
+# lock for creating certificate files
+SSL_CERT_LOCK = threading.RLock()
+
 
 class CustomEncoder(json.JSONEncoder):
     """ Helper class to convert JSON documents with datetime, decimals, or bytes. """
@@ -70,7 +80,7 @@ class CustomEncoder(json.JSONEncoder):
                 return float(o)
             else:
                 return int(o)
-        if isinstance(o, datetime):
+        if isinstance(o, (datetime, date)):
             return str(o)
         if isinstance(o, six.binary_type):
             return to_str(o)
@@ -80,29 +90,6 @@ class CustomEncoder(json.JSONEncoder):
             return None
 
 
-class FuncThread(threading.Thread):
-    """ Helper class to run a Python function in a background thread. """
-
-    def __init__(self, func, params=None, quiet=False):
-        threading.Thread.__init__(self)
-        self.daemon = True
-        self.params = params
-        self.func = func
-        self.quiet = quiet
-
-    def run(self):
-        try:
-            self.func(self.params)
-        except Exception:
-            if not self.quiet:
-                LOG.warning('Thread run method %s(%s) failed: %s' %
-                    (self.func, self.params, traceback.format_exc()))
-
-    def stop(self, quiet=False):
-        if not quiet and not self.quiet:
-            LOG.warning('Not implemented: FuncThread.stop(..)')
-
-
 class ShellCommandThread(FuncThread):
     """ Helper class to run a shell command in a background thread. """
 
@@ -173,6 +160,9 @@ class ShellCommandThread(FuncThread):
         # (Error: libc.musl-x86_64.so.1: cannot open shared object file)
         import psutil
 
+        if getattr(self, 'stopped', False):
+            return
+
         if not self.process:
             LOG.warning("No process found for command '%s'" % self.cmd)
             return
@@ -187,6 +177,7 @@ class ShellCommandThread(FuncThread):
         except Exception:
             if not quiet:
                 LOG.warning('Unable to kill process with pid %s' % parent_pid)
+        self.stopped = True
 
 
 class JsonObject(object):
@@ -214,8 +205,8 @@ class JsonObject(object):
         return result
 
     @classmethod
-    def from_json_list(cls, l):
-        return [cls.from_json(j) for j in l]
+    def from_json_list(cls, json_list):
+        return [cls.from_json(j) for j in json_list]
 
     @classmethod
     def as_dict(cls, obj):
@@ -301,18 +292,43 @@ class CaptureOutput(object):
         return threading.currentThread().ident
 
     def stdout(self):
-        return self._stdout.getvalue() if hasattr(self._stdout, 'getvalue') else self._stdout
+        return self._stream_value(self._stdout)
 
     def stderr(self):
-        return self._stderr.getvalue() if hasattr(self._stderr, 'getvalue') else self._stderr
+        return self._stream_value(self._stderr)
+
+    def _stream_value(self, stream):
+        return stream.getvalue() if hasattr(stream, 'getvalue') else stream
 
 
 # ----------------
 # UTILITY METHODS
 # ----------------
 
+def start_thread(method, *args, **kwargs):
+    thread = FuncThread(method, *args, **kwargs)
+    thread.start()
+    TMP_THREADS.append(thread)
+    return thread
 
-def is_string(s, include_unicode=True):
+
+def synchronized(lock=None):
+    """
+    Synchronization decorator as described in
+    http://blog.dscpl.com.au/2014/01/the-missing-synchronized-decorator.html.
+    """
+    def _decorator(wrapped):
+        @functools.wraps(wrapped)
+        def _wrapper(*args, **kwargs):
+            with lock:
+                return wrapped(*args, **kwargs)
+        return _wrapper
+    return _decorator
+
+
+def is_string(s, include_unicode=True, exclude_binary=False):
+    if isinstance(s, six.binary_type) and exclude_binary:
+        return False
     if isinstance(s, str):
         return True
     if include_unicode and isinstance(s, six.text_type):
@@ -324,12 +340,21 @@ def is_string_or_bytes(s):
     return is_string(s) or isinstance(s, six.string_types) or isinstance(s, bytes)
 
 
+def is_base64(s):
+    regex = r'^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=)?$'
+    return is_string(s) and re.match(regex, s)
+
+
 def md5(string):
     m = hashlib.md5()
     m.update(to_bytes(string))
     return m.hexdigest()
 
 
+def select_attributes(object, attributes):
+    return dict([(k, v) for k, v in object.items() if k in attributes])
+
+
 def in_docker():
     return config.in_docker()
 
@@ -342,25 +367,54 @@ def has_docker():
         return False
 
 
-def is_port_open(port_or_url, http_path=None, expect_success=True):
+def get_docker_container_names():
+    return bootstrap.get_docker_container_names()
+
+
+def path_from_url(url):
+    return '/%s' % str(url).partition('://')[2].partition('/')[2] if '://' in url else url
+
+
+def is_port_open(port_or_url, http_path=None, expect_success=True, protocols=['tcp']):
     port = port_or_url
     host = 'localhost'
     protocol = 'http'
+    protocols = protocols if isinstance(protocols, list) else [protocols]
     if isinstance(port, six.string_types):
         url = urlparse(port_or_url)
         port = url.port
         host = url.hostname
         protocol = url.scheme
-    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
-        sock.settimeout(1)
-        result = sock.connect_ex((host, port))
-        if result != 0:
-            return False
-    if not http_path:
+    nw_protocols = []
+    nw_protocols += ([socket.SOCK_STREAM] if 'tcp' in protocols else [])
+    nw_protocols += ([socket.SOCK_DGRAM] if 'udp' in protocols else [])
+    for nw_protocol in nw_protocols:
+        with closing(socket.socket(socket.AF_INET, nw_protocol)) as sock:
+            sock.settimeout(1)
+            if nw_protocol == socket.SOCK_DGRAM:
+                try:
+                    if port == 53:
+                        dnshost = '127.0.0.1' if host == 'localhost' else host
+                        resolver = dns.resolver.Resolver()
+                        resolver.nameservers = [dnshost]
+                        resolver.timeout = 1
+                        resolver.lifetime = 1
+                        answers = resolver.query('google.com', 'A')
+                        assert len(answers) > 0
+                    else:
+                        sock.sendto(bytes(), (host, port))
+                        sock.recvfrom(1024)
+                except Exception:
+                    return False
+            elif nw_protocol == socket.SOCK_STREAM:
+                result = sock.connect_ex((host, port))
+                if result != 0:
+                    return False
+    if 'tcp' not in protocols or not http_path:
         return True
     url = '%s://%s:%s%s' % (protocol, host, port, http_path)
     try:
-        response = safe_requests.get(url)
+        response = safe_requests.get(url, verify=False)
         return not expect_success or response.status_code < 400
     except Exception:
         return False
@@ -376,12 +430,21 @@ def wait_for_port_open(port, http_path=None, expect_success=True, retries=10, sl
     return retry(check, sleep=sleep_time, retries=retries)
 
 
-def get_free_tcp_port():
-    tcp = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    tcp.bind(('', 0))
-    addr, port = tcp.getsockname()
-    tcp.close()
-    return port
+def get_free_tcp_port(blacklist=None):
+    blacklist = blacklist or []
+    for i in range(10):
+        tcp = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        tcp.bind(('', 0))
+        addr, port = tcp.getsockname()
+        tcp.close()
+        if port not in blacklist:
+            return port
+    raise Exception('Unable to determine free TCP port with blacklist %s' % blacklist)
+
+
+def sleep_forever():
+    while True:
+        time.sleep(1)
 
 
 def get_service_protocol():
@@ -396,6 +459,12 @@ def timestamp(time=None, format=TIMESTAMP_FORMAT):
     return time.strftime(format)
 
 
+def timestamp_millis(time=None):
+    microsecond_time = timestamp(time=time, format=TIMESTAMP_FORMAT_MICROS)
+    # truncating microseconds to milliseconds, while leaving the "Z" indicator
+    return microsecond_time[:-4] + microsecond_time[-1]
+
+
 def retry(function, retries=3, sleep=1, sleep_before=0, **kwargs):
     raise_error = None
     if sleep_before > 0:
@@ -415,7 +484,7 @@ def dump_thread_info():
     print(run("ps aux | grep 'node\\|java\\|python'"))
 
 
-def merge_recursive(source, destination):
+def merge_recursive(source, destination, none_values=[None]):
     for key, value in source.items():
         if isinstance(value, dict):
             # get node or create one
@@ -425,7 +494,8 @@ def merge_recursive(source, destination):
             if not isinstance(destination, dict):
                 LOG.warning('Destination for merging %s=%s is not dict: %s' %
                     (key, value, destination))
-            destination[key] = value
+            if destination.get(key) in none_values:
+                destination[key] = value
     return destination
 
 
@@ -441,6 +511,20 @@ def merge_dicts(*dicts, **kwargs):
     return result
 
 
+def recurse_object(obj, func, path=''):
+    """ Recursively apply `func` to `obj` (may be a list, dict, or other object). """
+    obj = func(obj, path=path)
+    if isinstance(obj, list):
+        for i in range(len(obj)):
+            tmp_path = '%s[%s]' % (path or '.', i)
+            obj[i] = recurse_object(obj[i], func, tmp_path)
+    elif isinstance(obj, dict):
+        for k, v in obj.items():
+            tmp_path = '%s%s' % ((path + '.') if path else '', k)
+            obj[k] = recurse_object(v, func, tmp_path)
+    return obj
+
+
 def base64_to_hex(b64_string):
     return binascii.hexlify(base64.b64decode(b64_string))
 
@@ -456,16 +540,19 @@ def obj_to_xml(obj):
     return str(obj)
 
 
-def now_utc():
-    return mktime(datetime.utcnow())
+def now_utc(millis=False):
+    return mktime(datetime.utcnow(), millis=millis)
 
 
-def now():
-    return mktime(datetime.now())
+def now(millis=False):
+    return mktime(datetime.now(), millis=millis)
 
 
-def mktime(timestamp):
-    return time.mktime(timestamp.timetuple())
+def mktime(timestamp, millis=False):
+    if millis:
+        epoch = datetime.utcfromtimestamp(0)
+        return (timestamp - epoch).total_seconds()
+    return calendar.timegm(timestamp.timetuple())
 
 
 def mkdir(folder):
@@ -489,10 +576,21 @@ def ensure_readable(file_path, default_perms=None):
         os.chmod(file_path, default_perms)
 
 
-def chmod_r(path, mode):
-    """Recursive chmod"""
-    os.chmod(path, mode)
+def chown_r(path, user):
+    """ Recursive chown """
+    uid = pwd.getpwnam(user).pw_uid
+    gid = grp.getgrnam(user).gr_gid
+    os.chown(path, uid, gid)
+    for root, dirs, files in os.walk(path):
+        for dirname in dirs:
+            os.chown(os.path.join(root, dirname), uid, gid)
+        for filename in files:
+            os.chown(os.path.join(root, filename), uid, gid)
 
+
+def chmod_r(path, mode):
+    """ Recursive chmod """
+    os.chmod(path, mode)
     for root, dirnames, filenames in os.walk(path):
         for dirname in dirnames:
             os.chmod(os.path.join(root, dirname), mode)
@@ -506,6 +604,12 @@ def rm_rf(path):
     """
     if not path or not os.path.exists(path):
         return
+    # Running the native command can be an order of magnitude faster in Alpine on Travis-CI
+    if is_alpine():
+        try:
+            return run('rm -rf "%s"' % path)
+        except Exception:
+            pass
     # Make sure all files are writeable and dirs executable to remove
     chmod_r(path, 0o777)
     # check if the file is either a normal file, or, e.g., a fifo
@@ -530,6 +634,10 @@ def download(url, path, verify_ssl=True):
     # enable parallel file downloads during installation!
     s = requests.Session()
     r = s.get(url, stream=True, verify=verify_ssl)
+    # check status code before attempting to read body
+    if r.status_code >= 400:
+        raise Exception('Failed to download %s, response code %s' % (url, r.status_code))
+
     total = 0
     try:
         if not os.path.exists(os.path.dirname(path)):
@@ -545,26 +653,18 @@ def download(url, path, verify_ssl=True):
                     LOG.debug('Empty chunk %s (total %s) from %s' % (chunk, total, url))
             f.flush()
             os.fsync(f)
+        if os.path.getsize(path) == 0:
+            LOG.warning('Zero bytes downloaded from %s, retrying' % url)
+            download(url, path, verify_ssl)
+            return
+        LOG.debug('Done downloading %s, response code %s, total bytes %d' % (url, r.status_code, total))
     finally:
-        LOG.debug('Done downloading %s, response code %s' % (url, r.status_code))
         r.close()
         s.close()
 
 
-def parse_chunked_data(data):
-    """ Parse the body of an HTTP message transmitted with chunked transfer encoding. """
-    data = (data or '').strip()
-    chunks = []
-    while data:
-        length = re.match(r'^([0-9a-zA-Z]+)\r\n.*', data)
-        if not length:
-            break
-        length = length.group(1).lower()
-        length = int(length, 16)
-        data = data.partition('\r\n')[2]
-        chunks.append(data[:length])
-        data = data[length:].strip()
-    return ''.join(chunks)
+def first_char_to_lower(s):
+    return '%s%s' % (s[0].lower(), s[1:])
 
 
 def is_number(s):
@@ -575,10 +675,45 @@ def is_number(s):
         return False
 
 
+def is_mac_os():
+    return bootstrap.is_mac_os()
+
+
+def is_linux():
+    return bootstrap.is_linux()
+
+
+def is_alpine():
+    try:
+        if '_is_alpine_' not in CACHE:
+            CACHE['_is_alpine_'] = False
+            if not os.path.exists('/etc/issue'):
+                return False
+            out = to_str(subprocess.check_output('cat /etc/issue', shell=True))
+            CACHE['_is_alpine_'] = 'Alpine' in out
+    except subprocess.CalledProcessError:
+        return False
+    return CACHE['_is_alpine_']
+
+
+def get_arch():
+    if is_mac_os():
+        return 'osx'
+    if is_alpine():
+        return 'alpine'
+    if is_linux():
+        return 'linux'
+    raise Exception('Unable to determine system architecture')
+
+
 def short_uid():
     return str(uuid.uuid4())[0:8]
 
 
+def long_uid():
+    return str(uuid.uuid4())
+
+
 def json_safe(item):
     """ return a copy of the given object (e.g., dict) that is safe for JSON dumping """
     try:
@@ -602,6 +737,10 @@ def fix_json_keys(item):
     return item_copy
 
 
+def canonical_json(obj):
+    return json.dumps(obj, sort_keys=True)
+
+
 def save_file(file, content, append=False):
     mode = 'a' if append else 'w+'
     if not isinstance(content, six.string_types):
@@ -621,6 +760,17 @@ def load_file(file_path, default=None, mode=None):
     return result
 
 
+def get_or_create_file(file_path, content=None):
+    if os.path.exists(file_path):
+        return load_file(file_path)
+    content = '{}' if content is None else content
+    try:
+        save_file(file_path, content)
+        return content
+    except Exception:
+        pass
+
+
 def to_str(obj, encoding=DEFAULT_ENCODING, errors='strict'):
     """ If ``obj`` is an instance of ``binary_type``, return
     ``obj.decode(encoding, errors)``, otherwise return ``obj`` """
@@ -633,27 +783,62 @@ def to_bytes(obj, encoding=DEFAULT_ENCODING, errors='strict'):
     return obj.encode(encoding, errors) if isinstance(obj, six.text_type) else obj
 
 
+def str_insert(string, index, content):
+    """ Insert a substring into an existing string at a certain index. """
+    return '%s%s%s' % (string[:index], content, string[index:])
+
+
+def str_remove(string, index, end_index=None):
+    """ Remove a substring from an existing string at a certain from-to index range. """
+    end_index = end_index or (index + 1)
+    return '%s%s' % (string[:index], string[end_index:])
+
+
 def cleanup(files=True, env=ENV_DEV, quiet=True):
     if files:
         cleanup_tmp_files()
 
 
-def cleanup_threads_and_processes(quiet=True):
-    for t in TMP_THREADS:
-        t.stop(quiet=quiet)
-    for p in TMP_PROCESSES:
+def cleanup_threads_and_processes(quiet=True, debug=False):
+    for thread in TMP_THREADS:
+        if thread:
+            try:
+                print_debug('[shutdown] Cleaning up thread: %s' % thread, debug)
+                if hasattr(thread, 'shutdown'):
+                    thread.shutdown()
+                    continue
+                if hasattr(thread, 'kill'):
+                    thread.kill()
+                    continue
+                thread.stop(quiet=quiet)
+            except Exception as e:
+                print(e)
+    for proc in TMP_PROCESSES:
         try:
-            p.terminate()
+            print_debug('[shutdown] Cleaning up process: %s' % proc, debug)
+            proc.terminate()
         except Exception as e:
             print(e)
+    # clean up async tasks
+    try:
+        import asyncio
+        for task in asyncio.all_tasks():
+            try:
+                print_debug('[shutdown] Canceling asyncio task: %s' % task, debug)
+                task.cancel()
+            except Exception as e:
+                print(e)
+    except Exception:
+        pass
+    print_debug('[shutdown] Done cleaning up threads / processes / tasks', debug)
     # clear lists
     clear_list(TMP_THREADS)
     clear_list(TMP_PROCESSES)
 
 
-def clear_list(l):
-    while len(l):
-        del l[0]
+def clear_list(list_obj):
+    while len(list_obj):
+        del list_obj[0]
 
 
 def cleanup_tmp_files():
@@ -693,7 +878,11 @@ def is_zip_file(content):
     return zipfile.is_zipfile(stream)
 
 
-def unzip(path, target_dir):
+def unzip(path, target_dir, overwrite=True):
+    if is_alpine():
+        # Running the native command can be an order of magnitude faster in Alpine on Travis-CI
+        flags = '-o' if overwrite else ''
+        return run('cd %s; unzip %s %s' % (target_dir, flags, path))
     try:
         zip_ref = zipfile.ZipFile(path, 'r')
     except Exception as e:
@@ -701,9 +890,11 @@ def unzip(path, target_dir):
         raise e
     # Make sure to preserve file permissions in the zip file
     # https://www.burgundywall.com/post/preserving-file-perms-with-python-zipfile-module
-    for file_entry in zip_ref.infolist():
-        _unzip_file_entry(zip_ref, file_entry, target_dir)
-    zip_ref.close()
+    try:
+        for file_entry in zip_ref.infolist():
+            _unzip_file_entry(zip_ref, file_entry, target_dir)
+    finally:
+        zip_ref.close()
 
 
 def _unzip_file_entry(zip_ref, file_entry, target_dir):
@@ -716,21 +907,40 @@ def _unzip_file_entry(zip_ref, file_entry, target_dir):
     os.chmod(out_path, perm or 0o777)
 
 
-def is_jar_archive(content):
-    has_class_content = False
-    try:
-        has_class_content = 'class' in content
-    except TypeError:
-        # in Python 3 we need to use byte strings for byte-based file content
-        has_class_content = b'class' in content
-    if not has_class_content:
-        return False
+def untar(path, target_dir):
+    mode = 'r:gz' if path.endswith('gz') else 'r'
+    with tarfile.open(path, mode) as tar:
+        tar.extractall(path=target_dir)
+
+
+def zip_contains_jar_entries(content, jar_path_prefix=None, match_single_jar=True):
     try:
         with tempfile.NamedTemporaryFile() as tf:
             tf.write(content)
             tf.flush()
             with zipfile.ZipFile(tf.name, 'r') as zf:
-                zf.infolist()
+                jar_entries = [e for e in zf.infolist() if e.filename.lower().endswith('.jar')]
+                if match_single_jar and len(jar_entries) == 1 and len(zf.infolist()) == 1:
+                    return True
+                matching_prefix = [e for e in jar_entries if
+                    not jar_path_prefix or e.filename.lower().startswith(jar_path_prefix)]
+                return len(matching_prefix) > 0
+    except Exception:
+        return False
+
+
+def is_jar_archive(content):
+    """ Determine whether `content` contains valid zip bytes representing a JAR archive
+        that contains at least one *.class file and a META-INF/MANIFEST.MF file. """
+    try:
+        with tempfile.NamedTemporaryFile() as tf:
+            tf.write(content)
+            tf.flush()
+            with zipfile.ZipFile(tf.name, 'r') as zf:
+                class_files = [e for e in zf.infolist() if e.filename.endswith('.class')]
+                manifest_file = [e for e in zf.infolist() if e.filename.upper() == 'META-INF/MANIFEST.MF']
+                if not class_files or not manifest_file:
+                    return False
     except Exception:
         return False
     return True
@@ -741,31 +951,61 @@ def is_root():
     return out == 'root'
 
 
-def cleanup_resources():
+def cleanup_resources(debug=False):
     cleanup_tmp_files()
-    cleanup_threads_and_processes()
+    cleanup_threads_and_processes(debug=debug)
 
 
-def generate_ssl_cert(target_file=None, overwrite=False, random=False):
+def print_debug(msg, debug=False):
+    if debug:
+        print(msg)
+
+
+@synchronized(lock=SSL_CERT_LOCK)
+def generate_ssl_cert(target_file=None, overwrite=False, random=False, return_content=False, serial_number=None):
     # Note: Do NOT import "OpenSSL" at the root scope
     # (Our test Lambdas are importing this file but don't have the module installed)
     from OpenSSL import crypto
 
-    if os.path.exists(target_file):
-        key_file_name = '%s.key' % target_file
-        cert_file_name = '%s.crt' % target_file
-        return target_file, cert_file_name, key_file_name
+    def all_exist(*files):
+        return all([os.path.exists(f) for f in files])
+
+    def store_cert_key_files(base_filename):
+        key_file_name = '%s.key' % base_filename
+        cert_file_name = '%s.crt' % base_filename
+        # extract key and cert from target_file and store into separate files
+        content = load_file(target_file)
+        key_start = '-----BEGIN PRIVATE KEY-----'
+        key_end = '-----END PRIVATE KEY-----'
+        cert_start = '-----BEGIN CERTIFICATE-----'
+        cert_end = '-----END CERTIFICATE-----'
+        key_content = content[content.index(key_start): content.index(key_end) + len(key_end)]
+        cert_content = content[content.index(cert_start): content.rindex(cert_end) + len(cert_end)]
+        save_file(key_file_name, key_content)
+        save_file(cert_file_name, cert_content)
+        return cert_file_name, key_file_name
+
+    if target_file and not overwrite and os.path.exists(target_file):
+        key_file_name = ''
+        cert_file_name = ''
+        try:
+            cert_file_name, key_file_name = store_cert_key_files(target_file)
+        except Exception as e:
+            # fall back to temporary files if we cannot store/overwrite the files above
+            LOG.info('Error storing key/cert SSL files (falling back to random tmp file names): %s' % e)
+            target_file_tmp = new_tmp_file()
+            cert_file_name, key_file_name = store_cert_key_files(target_file_tmp)
+        if all_exist(cert_file_name, key_file_name):
+            return target_file, cert_file_name, key_file_name
     if random and target_file:
         if '.' in target_file:
             target_file = target_file.replace('.', '.%s.' % short_uid(), 1)
         else:
             target_file = '%s.%s' % (target_file, short_uid())
-    if target_file and not overwrite and os.path.exists(target_file):
-        return
 
     # create a key pair
     k = crypto.PKey()
-    k.generate_key(crypto.TYPE_RSA, 1024)
+    k.generate_key(crypto.TYPE_RSA, 2048)
 
     # create a self-signed cert
     cert = crypto.X509()
@@ -775,13 +1015,24 @@ def generate_ssl_cert(target_file=None, overwrite=False, random=False):
     subj.L = 'Some-Locality'
     subj.O = 'LocalStack Org'  # noqa
     subj.OU = 'Testing'
-    subj.CN = 'LocalStack'
-    cert.set_serial_number(1000)
+    subj.CN = 'localhost'
+    # Note: new requirements for recent OSX versions: https://support.apple.com/en-us/HT210176
+    # More details: https://www.iol.unh.edu/blog/2019/10/10/macos-catalina-and-chrome-trust
+    serial_number = serial_number or 1001
+    cert.set_version(2)
+    cert.set_serial_number(serial_number)
     cert.gmtime_adj_notBefore(0)
-    cert.gmtime_adj_notAfter(10 * 365 * 24 * 60 * 60)
+    cert.gmtime_adj_notAfter(2 * 365 * 24 * 60 * 60)
     cert.set_issuer(cert.get_subject())
     cert.set_pubkey(k)
-    cert.sign(k, 'sha1')
+    alt_names = b'DNS:localhost,DNS:test.localhost.atlassian.io,IP:127.0.0.1'
+    cert.add_extensions([
+        crypto.X509Extension(b'subjectAltName', False, alt_names),
+        crypto.X509Extension(b'basicConstraints', True, b'CA:false'),
+        crypto.X509Extension(b'keyUsage', True, b'nonRepudiation,digitalSignature,keyEncipherment'),
+        crypto.X509Extension(b'extendedKeyUsage', True, b'serverAuth')
+    ])
+    cert.sign(k, 'SHA256')
 
     cert_file = StringIO()
     key_file = StringIO()
@@ -791,17 +1042,30 @@ def generate_ssl_cert(target_file=None, overwrite=False, random=False):
     key_file_content = key_file.getvalue().strip()
     file_content = '%s\n%s' % (key_file_content, cert_file_content)
     if target_file:
-        save_file(target_file, file_content)
         key_file_name = '%s.key' % target_file
         cert_file_name = '%s.crt' % target_file
-        save_file(key_file_name, key_file_content)
-        save_file(cert_file_name, cert_file_content)
-        TMP_FILES.append(target_file)
-        TMP_FILES.append(key_file_name)
-        TMP_FILES.append(cert_file_name)
-        if random:
+        # check existence to avoid permission denied issues:
+        # https://github.com/localstack/localstack/issues/1607
+        if not all_exist(target_file, key_file_name, cert_file_name):
+            for i in range(2):
+                try:
+                    save_file(target_file, file_content)
+                    save_file(key_file_name, key_file_content)
+                    save_file(cert_file_name, cert_file_content)
+                    break
+                except Exception as e:
+                    if i > 0:
+                        raise
+                    LOG.info('Unable to store certificate file under %s, using tmp file instead: %s' % (target_file, e))
+                    # Fix for https://github.com/localstack/localstack/issues/1743
+                    target_file = '%s.pem' % new_tmp_file()
+                    key_file_name = '%s.key' % target_file
+                    cert_file_name = '%s.crt' % target_file
+            TMP_FILES.append(target_file)
+            TMP_FILES.append(key_file_name)
+            TMP_FILES.append(cert_file_name)
+        if not return_content:
             return target_file, cert_file_name, key_file_name
-        return file_content
     return file_content
 
 
@@ -817,75 +1081,17 @@ def run_cmd_safe(**kwargs):
     return run_safe(run, print_error=False, **kwargs)
 
 
-def run(cmd, cache_duration_secs=0, print_error=True, asynchronous=False, stdin=False,
-        stderr=subprocess.STDOUT, outfile=None, env_vars=None, inherit_cwd=False,
-        inherit_env=True, tty=False):
-    # don't use subprocess module inn Python 2 as it is not thread-safe
-    # http://stackoverflow.com/questions/21194380/is-subprocess-popen-not-thread-safe
-    if six.PY2:
-        import subprocess32 as subprocess
-    else:
-        import subprocess
-
-    env_dict = os.environ.copy() if inherit_env else {}
-    if env_vars:
-        env_dict.update(env_vars)
-
-    if tty:
-        asynchronous = True
-        stdin = True
+def run(cmd, cache_duration_secs=0, **kwargs):
 
     def do_run(cmd):
-        try:
-            cwd = os.getcwd() if inherit_cwd else None
-            if not asynchronous:
-                if stdin:
-                    return subprocess.check_output(cmd, shell=True, stderr=stderr, env=env_dict,
-                        stdin=subprocess.PIPE, cwd=cwd)
-                output = subprocess.check_output(cmd, shell=True, stderr=stderr, env=env_dict, cwd=cwd)
-                return output.decode(DEFAULT_ENCODING)
-
-            # subprocess.Popen is not thread-safe, hence use a mutex here.. (TODO: mutex still needed?)
-            with mutex_popen:
-                stdin_arg = subprocess.PIPE if stdin else None
-                stdout_arg = open(outfile, 'wb') if isinstance(outfile, six.string_types) else outfile
-                stderr_arg = stderr
-                if tty:
-                    master_fd, slave_fd = pty.openpty()
-                    stdin_arg = slave_fd
-                    stdout_arg = stderr_arg = None
-
-                # start the actual sub process
-                process = subprocess.Popen(cmd, shell=True, stdin=stdin_arg, bufsize=-1,
-                    stderr=stderr_arg, stdout=stdout_arg, env=env_dict, cwd=cwd, preexec_fn=os.setsid)
-
-                if tty:
-                    # based on: https://stackoverflow.com/questions/41542960
-                    def pipe_streams(*args):
-                        while process.poll() is None:
-                            r, w, e = select.select([sys.stdin, master_fd], [], [])
-                            if sys.stdin in r:
-                                d = os.read(sys.stdin.fileno(), 10240)
-                                os.write(master_fd, d)
-                            elif master_fd in r:
-                                o = os.read(master_fd, 10240)
-                                if o:
-                                    os.write(sys.stdout.fileno(), o)
-
-                    FuncThread(pipe_streams).start()
-
-                return process
-        except subprocess.CalledProcessError as e:
-            if print_error:
-                print("ERROR: '%s': exit code %s; output: %s" % (cmd, e.returncode, e.output))
-                sys.stdout.flush()
-            raise e
+        return bootstrap.run(cmd, **kwargs)
 
     if cache_duration_secs <= 0:
         return do_run(cmd)
 
     hash = md5(cmd)
     cache_file = CACHE_FILE_PATTERN.replace('*', hash)
+    mkdir(os.path.dirname(CACHE_FILE_PATTERN))
     if os.path.isfile(cache_file):
         # check file age
         mod_time = os.path.getmtime(cache_file)
@@ -907,6 +1113,10 @@ def clone(item):
     return json.loads(json.dumps(item))
 
 
+def clone_safe(item):
+    return clone(json_safe(item))
+
+
 def remove_non_ascii(text):
     # text = unicode(text, "utf-8")
     text = text.decode('utf-8', CODEC_HANDLER_UNDERSCORE)
@@ -979,7 +1189,15 @@ def clean_cache(file_pattern=CACHE_FILE_PATTERN,
 
 
 def truncate(data, max_length=100):
-    return (data[:max_length] + '...') if len(data) > max_length else data
+    return ('%s...' % data[:max_length]) if len(data or '') > max_length else data
+
+
+def escape_html(string, quote=False):
+    if six.PY2:
+        import cgi
+        return cgi.escape(string, quote=quote)
+    import html
+    return html.escape(string, quote=quote)
 
 
 def parallelize(func, list, size=None):
@@ -999,3 +1217,7 @@ def isoformat_milliseconds(t):
         return t.isoformat(timespec='milliseconds')
     except TypeError:
         return t.isoformat()[:-3]
+
+
+# Code that requires util functions from above
+CACHE_FILE_PATTERN = CACHE_FILE_PATTERN.replace('_random_dir_', short_uid())
